# Generated by roxygen2: do not edit by hand

export(as.yaml)
export(read_yaml)
export(verbatim_logical)
export(write_yaml)
export(yaml.load)
export(yaml.load_file)
useDynLib(yaml, .registration = TRUE, .fixes = "C_")
