#' @keywords internal
"_PACKAGE"

#' @useDynLib yaml, .registration = TRUE, .fixes = "C_"
NULL

## usethis namespace: start
## usethis namespace: end

## mockable bindings: start
## mockable bindings: end
NULL
